
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripList(name)
{
	this.name = "";
	this.xpaths = {};
	this.length = 0;

	this.setName = function(name)
	{
		if (name) this.name = "" + name;
	}

	this.add = function(xpath)
	{
		if (!xpath) return;
		if (!this.xpaths[xpath.value]) this.length++;
		this.xpaths[xpath.value] = xpath;
	}

	this.remove = function(xValue)
	{
		if (!xValue || !this.xpaths[xValue]) return;
		this.xpaths[xValue] = null;
		this.length--;
	}

	this.reset = function()
	{
		this.xpaths = {};
		this.length = 0;
	}

	this.sortXPaths = function(reduce)
	{
		var xArr = [];
		for (var v in this.xpaths) if (this.xpaths[v]) xArr.push(this.xpaths[v]);
		xArr.sort(this.byName);

		if (reduce)
		{
			var rArr = [];
			var tArr = [];
			var xpath = xArr.shift();
			while (xpath)
			{
				rArr.push(xpath);
				for (var i = 0; i < xArr.length; i++)
				{
					if (
						xArr[i].value.indexOf(xpath.value) != 0 ||       // does not contain
						xArr[i].value.charAt(xpath.value.length) != "/"  // is not the end of a sub-path
					) tArr.push(xArr[i]);
				}
				xArr = tArr;
				tArr = [];
				xpath = xArr.shift();
			}
			xArr = rArr;
			this.reset();
		}

		for (var i = 0; i < xArr.length; i++) if (xArr[i]) this.add(xArr[i]);
	}

	this.clone = function(purge)
	{
		var tmp = new this.constructor(this.name);
		for (var k in this) if(this[k] && typeof(this[k]) != "object" && !k.match(/name|length/)) tmp[k] = this[k];
		for (var v in this.xpaths) if (this.xpaths[v]) tmp.add(this.xpaths[v].clone(purge));
		return tmp;
	}

	this.generateXml = function()
	{
		var tmp = "";
		for (var v in this.xpaths)
		{
			if (!this.xpaths[v]) continue;
			tmp += this.xpaths[v].generateXml();
		}
		if (tmp != "") return "\t\t<" + this.name + ">\n" + tmp + "\t\t</" + this.name + ">\n";
		else return "";
	}

	this.byName = function(a, b)
	{
		a = a.value;
		b = b.value;
		if (a.length < b.length)
		{
			if (b.indexOf(a) == 0) return -1;
			if (a < b) return -1;
			if (a > b) return 1;
			return 0;
		}
		if (a.length > b.length)
		{
			if (a.indexOf(b) == 0) return 1;
			if (a < b) return -1;
			if (a > b) return 1;
			return 0;
		}
		if (a < b) return -1;
		if (a > b) return 1;
		return 0;
	}

	this.setName(name);
}

function YaripWhitelist()
{
	this.name = "whitelist";
	this.xpaths = {};
	this.length = 0;
}
YaripWhitelist.prototype = new YaripList;
YaripWhitelist.prototype.getXPathsArray = function(tab)
{
	var arr = [];
	for (var v in this.xpaths)
	{
		var xpath = this.xpaths[v];
		if (xpath) arr[arr.length] = [v, xpath.getDeep(), xpath.getFound(), xpath.getNotFound(), xpath.getWhitelisted()];
	}
	return arr;
}

function YaripBlacklist()
{
	this.name = "blacklist";
	this.xpaths = {};
	this.length = 0;
}
YaripBlacklist.prototype = new YaripList;
YaripBlacklist.prototype.getXPathsArray = function(tab)
{
	var arr = [];
	for (var v in this.xpaths)
	{
		var xpath = this.xpaths[v];
		if (xpath) arr[arr.length] = [v, xpath.getDeep(), xpath.getForce(), xpath.getDependsOnWhitelist(), xpath.getFound(), xpath.getNotFound(), xpath.getWhitelisted()];
	}
	return arr;
}
YaripBlacklist.prototype.generateCSS = function()
{
	var tmp = "";
	for (var v in this.xpaths)
	{
		if (!this.xpaths[v]) continue;
		tmp += this.xpaths[v].generateCSS();
	}
	if (tmp != "") return tmp.replace(/,(\n)$/, "$1") + "\t{\n\t\tdisplay: none !important;\n\t}\n";
	else return "";
}

function YaripStylesList()
{
	this.name = "styles";
	this.xpaths = {};
	this.length = 0;
}
YaripStylesList.prototype = new YaripList;
YaripStylesList.prototype.getXPathsArray = function(tab)
{
	var arr = [];
	for (var v in this.xpaths)
	{
		var xpath = this.xpaths[v];
		if (xpath) arr[arr.length] = [v, xpath.style, xpath.getDeep(), xpath.getFound(), xpath.getNotFound()];
	}
	return arr;
}
YaripStylesList.prototype.sortXPaths = function()
{
	var xArr = [];
	for (var v in this.xpaths) if (this.xpaths[v]) xArr.push(this.xpaths[v]);
	xArr.sort(this.byName);
	this.reset();
	for (var i = 0; i < xArr.length; i++) if (xArr[i]) this.add(xArr[i]);
}
YaripStylesList.prototype.generateCSS = function()
{
	var tmp = "";
	for (var v in this.xpaths)
	{
		if (!this.xpaths[v]) continue;
		var xpathAsStyle = this.xpaths[v].xpathToStyle();
		if (xpathAsStyle) tmp += "\t" + xpathAsStyle + "\n\t{\n\t\t" + this.xpaths[v].style.replace(/\s*;(\s*\*\/)?\s*/g, ";$1\n\t\t").replace(/\n\t\t$/, "") + "\n\t}\n";
	}
	return tmp;
}

function YaripExtList()
{
	this.name = "extension";
	this.xpaths = {};
	this.length = 0;
}
YaripExtList.prototype = new YaripList;
YaripExtList.prototype.getXPathsArray = function()
{
	var arr = [];
	for (var id in this.xpaths) if (this.xpaths[id]) arr[arr.length] = [id];
	return arr;
}
YaripExtList.prototype.sortXPaths = function() {}
