
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripLoader()
{
	this.hasWhitelist = false;
	this.hasBlacklist = false;
	this.hasStyles = false;

	this.load = function(doc)
	{
		if (!doc || doc.nodeName != "#document") return false;

		var pageAddress = yarip.getAddress(doc, yarip.PAGE_MODE);
		if (!pageAddress) return false;

		var aAddress = yarip.getAddressArray(doc);

		this.doWhitelisting(doc, pageAddress, aAddress);
		this.doBlacklisting(doc, pageAddress, aAddress);
		this.doStyling(doc, pageAddress, aAddress);
		this.doExtensioning(doc, pageAddress, aAddress);

		if (this.hasWhitelist && !this.hasBlacklist) this.removeAllExceptWhitelisted(doc, pageAddress);

		var found = this.hasWhitelist || this.hasBlacklist || this.hasStyles;
		this.hasWhitelist = false;
		this.hasBlacklist = false;
		this.hasStyles = false;
		return found;
	}

	this.doWhitelisting = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var whitelist = yarip.map.obj[a].whitelist;

			for (var v in whitelist.xpaths)
			{
				var xpath = whitelist.xpaths[v];

				if (!xpath || !xpath.deep && !this.equalAddresses(pageAddress, a)) continue;

				if (yarip.whitelist(doc, a, v))
				{
					whitelist.xpaths[v].incrementFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
					this.hasWhitelist = true;
				}
				else
				{
					whitelist.xpaths[v].incrementNotFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
				}
			}
		}
	}

	this.doBlacklisting = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var blacklist = yarip.map.obj[a].blacklist;

			for (var v in blacklist.xpaths)
			{
				var xpath = blacklist.xpaths[v];

				if (!xpath || !xpath.force && !xpath.deep && !this.equalAddresses(pageAddress, a)) continue;
				if (!this.hasWhitelist && xpath.getDependsOnWhitelist()) continue;

				if (yarip.blacklist(doc, a, v, false, xpath.force))
				{
					blacklist.xpaths[v].incrementFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
					/*if (!v.match(/@[a-z]+$/i))*/ this.hasBlacklist = true;
				}
				else
				{
					blacklist.xpaths[v].incrementNotFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
				}
			}
		}
	}

	this.doStyling = function(doc, pageAddress, aAddress)
	{
		for (var a in aAddress)
		{
			var styles = yarip.map.obj[a].styles;

			for (var v in styles.xpaths)
			{
				var xpath = styles.xpaths[v];

				if (!xpath || !xpath.deep && !this.equalAddresses(pageAddress, a)) continue;

				if (yarip.style(doc, a, v, xpath.getStyle()))
				{
					styles.xpaths[v].incrementFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
					this.hasStyles = true;
				}
				else
				{
					styles.xpaths[v].incrementNotFound();
					yarip.setChangeType(yarip.MINOR_CHANGE);
				}
			}
		}
	}

	this.doExtensioning = function(doc, pageAddress, aAddress)
	{
		var aExtAddress = yarip.getExtensionAddressArray(aAddress);
		if (!aExtAddress) return;

		this.doWhitelisting(doc, pageAddress, aExtAddress);
		this.doBlacklisting(doc, pageAddress, aExtAddress);
		this.doStyling(doc, pageAddress, aExtAddress);
	}

	this.removeAllExceptWhitelisted = function(doc, pageAddress)
	{
		yarip.whitelist(doc, pageAddress, "/html/head/descendant-or-self::*[not(@status='blacklisted')]"); // whitelist the head and descendants (stylesheets)
		yarip.blacklist(doc, pageAddress, "//*[not(@status='whitelisted')]"); // remove all except whitelisted
	}

	this.equalAddresses = function(pageAddress, address)
	{
		if (pageAddress.indexOf(address) == -1) return true; // ignore deep (extension)

		pageAddress = pageAddress.replace(/[/]+$/, "");
		address = address.replace(/[/]+$/, "");
		if (address.match(yarip.noSchemeRegExp)) pageAddress = pageAddress.replace(/^[^:/?#]+:[/]{2}/, ""); // remove scheme
		return pageAddress == address;
	}
}
